#!/usr/bin/perl

# Given file or directory, print out the relevant hash entries.
# This needs to locate a matching hash file, decide if it's possibly
# up to date. If a file is modified after the date indicated in the
# hash file, nothing is returned.
# You hand in the base directory first, for locating the hashes and
# establishing the relative paths. If no file/dir follows, the freshest
# hash data found is printed. For everything.

use File::Basename qw(basename dirname);
use File::Spec::Functions qw(abs2rel rel2abs);
use Cwd qw(cwd realpath);

use FindBin qw($Bin);
use lib $Bin;
use TreeHasher;

use strict;

if(@ARGV < 2)
{
	print "Usage:\n\t$0 <base> [directories/files to get hashes for]\n\n"
	.	"This extracts hashes from the tree indicated by the base for the "
	.	"given files/directories. The output is supposed to be passed to a "
	.	"hash sum verification program (for $hash_alg, most probably)."
	.	"\n";
	exit(1);
}

TreeHasher::init(shift @ARGV);

# Query for directories or files.
for my $q (@ARGV)
{
	# Using realpath here to resolve symlinks.
	# Might provide an option to be able to look for non-existing files.
	my $realpath = realpath($q);
	#print STDERR "realpath: $realpath\n";
	my $relpath = abs2rel($realpath, $base);
	# Only consider directories inside the base.
	die "$0: outsider: $relpath\n"
		if $relpath =~ m,^..(/|$),;
	if(-f $realpath)
	{
		# Only one hash file needed to get the single entry.
		find_file($relpath, $q);
	}
	elsif(-d $realpath)
	{
		# A directory wants hashes of itself and its subdirectories.
		# Recursion ensues.
		find_dir($relpath, $q);
	}
	else
	{
		print STDERR "$0: no file and no directory: $realpath\n";
	}
}

sub find_file
{
	my $relpath = shift;
	my $wantpath = shift;
	my $file = basename($relpath);
	my $dir  = dirname($relpath);

	my $hashfile = last_dirhash($dir);
	if(defined $hashfile)
	{
		#print STDERR "found: $hashfile->{hashpath} (rel: $ckbase/$relpath)\n";
		my $lines = get_hashlist($hashfile->{hashpath});
		my $hreldir = abs2rel($ckbase.'/'.$dir, $hashfile->{dirpath}).'/';
		for(@{$lines})
		{
			if($_->{dir} eq $hreldir and $_->{file} eq $file)
			{
				print hashline($_, $wantpath, '');
				return;
			}
		}
	}
}

my $reclevel = 0;
sub find_dir
{
	if(++$reclevel > $reclimit)
	{
		die "$0: recursion limit of $reclimit reached\n"
	}
	my $relpath = shift;
	my $wantpath = shift;
	$wantpath .= '/'
		unless $wantpath =~ m:/$:;

	my $realpath = $base.'/'.$relpath;
#print STDERR "find: $realpath\n";
	for my $d (@{dirs($realpath)})
	{
#print STDERR "subdir: $d\n";
		my $relsubdir = $relpath eq '.' ? $d : $relpath.'/'.$d;
		find_dir($relsubdir, $wantpath.$d);
	}

	# Test things just for this dir, without recursion.
	# This could be older hashes than available for children.
	my $hashfile = last_dirhash($relpath);
	if(defined $hashfile)
	{
#print STDERR "hashpath: $hashfile->{hashpath}\n";
		# Path confusion. Need to prefix files properly.
		# There is a prefix if the hashfile is a parent of the direct one.
		my $hashprefix = abs2rel($ckbase.'/'.$relpath, $hashfile->{dirpath}).'/';
		# Read contents of hashfile into memory, possibly pulling from the cache.
		my $lines = get_hashlist($hashfile->{hashpath});
		for(@{$lines})
		{
#print STDERR "dir: $_->{dir} vs. $hashprefix\n";
			if($_->{dir} eq $hashprefix)
			{
				print hashline($_, $wantpath);
			}
		}
# timing: test impact of cleared cache
#		%cache = ();
	}
#print STDERR "done: $realpath\n";
	--$reclevel;
}
