#!/bin/bash

urls=.urls.txt
files=.files.txt
fileurls=.file-urls.sh
blacklist=.fileblacklist.txt
urlblacklist=.urlblacklist.txt
auth=.auth.txt

PATH=$(dirname $0):$PATH
NICEWAIT=${NICEWAIT:-0}

# There is a file blacklist and an URL blacklist.
# Once a file is blacklisted, no URL for it will be tried.
# If an URL is blacklisted, we may still try to get the file
# from elsewhere.
# If a definite list of URLs exists, URL blacklisting is used.
# Otherwise (file URLs constructed, e.g. for a mirroring system),
# the file itself is blacklisted.

blacklisted() # (file, url)
{
  grep -q -Fx "$1" "$blacklist" 2>/dev/null ||
  grep -q -Fx "$2" "$urlblacklist" 2>/dev/null
}

blacklist() # (file, url)
{
  echo "blacklisting: $1 ($2)"
  blacklisted "$1" ||
  if test -e "$urls"; then
    echo "$2" >> "$urlblacklist"
  else
    echo "$1" >> "$blacklist"
  fi  
}

handle_file()
{
  file=$1
  url=$2

  # In future, the url files and associated scripts shall be hidden with dot,
  # and files with "-" just complicate command lines.
  case "$file" in
    -*)
      echo "ignoring file that starts with hyphen: $file"
      return
    ;;
    .*)
      echo "ignoring file that starts with a dot: $file"
      return
    ;;
  esac

  if blacklisted "$file" "$url"; then
    echo "blacklisted: $file ($url)"
    return
  fi

    # Only existing files with some content count. I will take care of
    # legitimate cases of empty files later. For now they are accidents.
    if test -e "$file" && test $(stat -c %s "$file") -gt 0; then
       # echo "already there: $file"
       true
    else
      path=$(locate-file "$file")
      if test $? == 0 && test $(stat -c %s -- "$path") -gt 0 &&
         test $(basename "$path") = "$file"; then
        echo "found elsewhere: $path"
        if test "$(basename "$(dirname "$path")")" = _unsorted; then
          echo "moving it from unsorted area"
          mv -v "$path" .
        else
          echo "linking in $(relpath "$path")"
          ln -sv $(relpath "$path") .
        fi
      else
        echo "fetching: $file" # Paranoid for now, deleting on failure.
        mkdir -p $(dirname "$file")
	if ! $first; then
          sleep "$NICEWAIT"
        fi
        
        # Try another source mirror first, if configured.
        tfile=$(mktemp ".$file.tmp.XXXX")
        chmod 644 "$tfile"
	if test -n "$TRY_MIRROR"; then
	  wget --quiet --no-check-certificate --timeout=23 --tries=1 -O "$tfile" "$TRY_MIRROR/$file" &&
          mv "$tfile" "$file"
        fi
        if test ! -e "$file"; then
	  if test -e "$auth"; then
            wget \
              --user="$(head -n 1 < $auth)" --password="$(head -n 2 < $auth | tail -n 1)" \
              --quiet --no-check-certificate --timeout=23 --tries=2 -O "$tfile" "$url"
          else
            wget --quiet --no-check-certificate --timeout=23 --tries=2 -O "$tfile" "$url"
          fi &&
          mv "$tfile" "$file" || { rm -f "$tfile"; blacklist "$file" "$url"; }
        fi
        first=false
      fi
    fi

}

# Does first handling really work? Subshell?
first=true
for dir in "$@"
do
  echo "dir: $dir"
  pushd "$dir"
  
  if test -e "$urls"; then

    while IFS=$'\t' read -r file url
    do
      handle_file "$file" "$url"
    done < <(cat "$urls" | grep -v '^#')

  elif test -e "$files"; then

    if test -e "$fileurls"; then
      while read -r file
      do
        handle_file "$file" "$(sh $fileurls "$file" | head -n 1)"
      done < <(cat "$files" | grep -v '^#')
    else
      echo "$0: error: $fileurls does not exist in: $dir ($PWD)" >&2
    fi

  else

    echo "$0: dunno what to do in: $dir" >&2

  fi

  popd
done
